import { enrichReducedReportWithBaseline } from '../../collect/utils/report/utils.js';
import { table } from '../../../core/md/table.js';
import { style } from '../../../core/md/font-style.js';
import { headline } from '../../../core/md/headline.js';
import { NEW_LINE } from '../../../core/md/constants.js';
import { details } from '../../../core/md/details.js';
const budgetsSymbol = '🔒';
export async function generateMdReport(flowResult) {
    const name = flowResult.name;
    const dateTime = `Date/Time: ${style(new Date().toISOString().replace('T', ' ').split('.')[0].slice(0, -3))}  `;
    const stepsTable = await getStepsTable(flowResult);
    let md = `${headline(name)}${NEW_LINE}
${dateTime}${NEW_LINE}
${stepsTable}${NEW_LINE}
`;
    const budgetsTable = await getBudgetTable(flowResult);
    if (budgetsTable !== '') {
        md += details(`${budgetsSymbol} Budgets`, budgetsTable) + NEW_LINE;
    }
    return md;
}
/**
 * ## Navigation report (127.0.0.1/)
 *
 * | Resource Type | Requests | Transfer Size | Over Budget |
 * | :-----------: | :------: | :-----------: | :---------: |
 * |     Total     |  205774  |     total     |   204750    |
 * |    Script     |  101958  |    script     |      -      |
 * |  Third-party  |  97308   |  third-party  |      -      |
 *
 * |         Metric         | Measurement | Over Budget |
 * | :--------------------: | :---------: | :---------: |
 * | First Meaningful Paint |    2043     |     43      |
 * |  Time to Interactive   |    4745     |      -      |
 *
 */
export async function getBudgetTable(reducedReport, options = { heading: 3 }) {
    const performanceBudgets = reducedReport.steps
        .filter(({ resourceCountsBudget, resourceSizesBudget, timingsBudget }) => resourceCountsBudget || resourceSizesBudget || timingsBudget)
        .map(({ name, resourceCountsBudget, resourceSizesBudget, timingsBudget }) => ({
        name,
        resultsSizeBudget: getResourceSizes(resourceSizesBudget),
        resultsCountBudget: getResourceCounts(resourceCountsBudget),
        resultsTimingBudget: getTimings(timingsBudget)
    }));
    return performanceBudgets.length ? (await Promise.all(performanceBudgets.map(async (b) => {
        let md = headline(b.name, options.heading) + NEW_LINE + NEW_LINE;
        if (b.resultsSizeBudget !== undefined) {
            md += style('Resource Size Budget') + NEW_LINE + NEW_LINE;
            md += await table(b.resultsSizeBudget) + NEW_LINE + NEW_LINE;
        }
        if (b.resultsCountBudget !== undefined) {
            md += style('Resource Count Budget') + NEW_LINE + NEW_LINE;
            md += await table(b.resultsCountBudget) + NEW_LINE + NEW_LINE;
        }
        if (b.resultsTimingBudget !== undefined) {
            md += style('Timing Budget') + NEW_LINE + NEW_LINE;
            md += await table(b.resultsTimingBudget) + NEW_LINE + NEW_LINE;
        }
        return md;
    }))).join(NEW_LINE) : '';
}
function getTimings(resultsTimingBudget) {
    if (resultsTimingBudget === undefined) {
        return undefined;
    }
    else {
        return [
            // [ {text: string, ...props } ]
            resultsTimingBudget.headings.map((h) => h.text),
            ...resultsTimingBudget.items.map(({ label, measurement, overBudget }) => {
                const row = [];
                if (label === 'Cumulative Layout Shift') {
                    return [label + '',
                        measurement === undefined ? '-' : parseFloat(measurement.value).toFixed(2),
                        overBudget === undefined ? '-' : parseFloat(overBudget).toFixed(2),
                    ];
                }
                else {
                    return [label + '',
                        measurement === undefined ? '-' : measurement + ' ms',
                        overBudget === undefined ? '-' : overBudget + ' ms'
                    ];
                }
            })
        ];
    }
}
;
function getResourceSizes(resourceSizesBudget) {
    if (resourceSizesBudget === undefined) {
        return undefined;
    }
    else {
        return [
            resourceSizesBudget.headings
                .filter((i) => ['label', 'transferSize', 'sizeOverBudget'].includes(i.key))
                .map((h) => h.text),
            ...resourceSizesBudget.items
                .map(({ label, transferSize, sizeOverBudget }) => [
                label + '',
                formatBytes(transferSize),
                sizeOverBudget ? formatBytes(sizeOverBudget) : '-'
            ])
        ];
    }
}
function getResourceCounts(resourceCountsBudget) {
    if (resourceCountsBudget === undefined) {
        return undefined;
    }
    else {
        return [
            resourceCountsBudget.headings
                .filter((i) => ['label', 'requestCount', 'countOverBudget'].includes(i.key))
                .map((h) => h.key === 'countOverBudget' ? 'Over Budget' : h.text),
            ...resourceCountsBudget.items
                .map(({ label, requestCount, countOverBudget }) => [
                label + '',
                requestCount + '',
                countOverBudget ? countOverBudget : '-'
            ])
        ];
    }
}
/**
 * | Step Name       | Gather Mode |Performance | Accessibility | BestPractices | Seo  | PWA |
 * | --------------- | ----------- | ------------- | ------------- | ---- | --- |
 * |  Nav1           |  99         | 50            | 100           | 98   |  -  |
 * |  Snap   1       |  3/3        | 22/5          | 5/2           | 7/10 |  -  |
 * |  TimeSpan 1     |  10/11      | -             | 4/7           | 7/10 |  -  |
 */
export async function getStepsTable(reducedReport, baselineResults) {
    const reportCategories = Object.keys(reducedReport.steps[0].results);
    const tableStepsArr = formatStepsForMdTable(reportCategories, reducedReport, baselineResults);
    const alignOptions = headerAlignment(reportCategories);
    const tableArr = extractTableArr(reportCategories, tableStepsArr);
    return await table(tableArr, alignOptions);
}
function formatStepsForMdTable(reportCategories, reducedReport, baselineResults) {
    if (baselineResults) {
        const enrichedReducedReport = enrichReducedReportWithBaseline(reducedReport, baselineResults);
        return enrichedReducedReport.steps.map((step) => {
            const results = reportCategories.map(category => extractEnrichedResults(step, category));
            return [step.name, step.gatherMode].concat(results);
        });
    }
    return reducedReport.steps.map((step) => {
        const results = reportCategories.map(category => extractResultsValue(step.results[category]));
        // add `budgetsSymbol` to gatherMode to indicate budgets
        step.resourceCountsBudget && (step.gatherMode = step.gatherMode + ` ${budgetsSymbol}`);
        return [step.name, step.gatherMode].concat(results);
    });
}
function formatBytes(bytes, decimals = 2) {
    if (!+bytes)
        return '0 Bytes';
    const k = 1024;
    const dm = decimals < 0 ? 0 : decimals;
    const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
    const i = Math.floor(Math.log(bytes) / Math.log(k));
    return `${parseFloat((bytes / Math.pow(k, i)).toFixed(dm))} ${sizes[i]}`;
}
function extractTableArr(reportCategories, steps) {
    const tableHead = extractTableHead(reportCategories);
    return [tableHead].concat(steps);
}
function extractTableHead(reportCategories) {
    return ['Step Name', 'Gather Mode']
        .concat(reportCategories.map(c => c.split('-').map(cN => cN[0].toUpperCase() + cN.slice(1)).join(' ')));
}
function extractFractionalResultValue(fractionResults) {
    const { totalWeight, numPassed, numPassableAudits } = fractionResults;
    const value = numPassed + '/' + numPassableAudits;
    return totalWeight === 0 ? 'Ø ' + value : value;
}
function extractResultsValue(stepResult) {
    if (typeof stepResult === 'number') {
        return (stepResult * 100).toFixed(0);
    }
    return stepResult ? extractFractionalResultValue(stepResult) : '-';
}
function extractEnrichedResults(step, category) {
    const result = extractResultsValue(step.results[category]);
    const baseline = extractResultsValue(step.baseline[category]);
    return resultWithBaselineComparison(result, baseline);
}
function resultWithBaselineComparison(result, baseline) {
    if (result === baseline) {
        return result;
    }
    const resultNum = Number(result.replace('Ø ', '').split('/')[0]);
    const baselineNum = Number(baseline.replace('Ø ', '').split('/')[0]);
    const difference = baselineNum - resultNum;
    return `${result} (${difference > 0 ? '+' : ''}${difference})`;
}
function headerAlignment(reportCategories) {
    const reportFormats = reportCategories.map(_ => 'c');
    return ['l', 'c'].concat(reportFormats);
}
//# sourceMappingURL=md-report.js.map